;;; crepulsion.el --- repulsion between Kana/Kanji characters and others
;; $Id: crepulsion.el,v 1.3 2006/03/10 01:42:03 ken Exp $

;; Copyright (c) 2006 KAMADA Ken'ichi.
;; All rights reserved.

;; Redistribution and use in source and binary forms, with or without
;; modification, are permitted provided that the following conditions
;; are met:
;; 1. Redistributions of source code must retain the above copyright
;;    notice, this list of conditions and the following disclaimer.
;; 2. Redistributions in binary form must reproduce the above copyright
;;    notice, this list of conditions and the following disclaimer in the
;;    documentation and/or other materials provided with the distribution.

;; THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
;; ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
;; IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
;; ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
;; FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
;; DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
;; OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
;; HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
;; LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
;; OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
;; SUCH DAMAGE.


(defvar crepulsion-mode nil
  "Non-nil if in crepulusion minor mode.")
(make-variable-buffer-local 'crepulsion-mode)

(defvar crepulsion-mode-line nil)
(make-variable-buffer-local 'crepulsion-mode-line)


(setq-default crepulsion-mode-line " CRepul")
(if (not (assoc 'crepulsion-mode minor-mode-alist))
    (setq minor-mode-alist (cons '(crepulsion-mode crepulsion-mode-line)
				 minor-mode-alist)))


(defun crepulsion-mode (&optional arg)
  (interactive "P")
  (setq crepulsion-mode
	(if arg
	    (> (prefix-numeric-value arg) 0)
	  (not crepulsion-mode)))
  (if crepulsion-mode
      (if (not (memq 'crepulsion-change-func after-change-functions))
	  (setq after-change-functions
		(cons 'crepulsion-change-func after-change-functions)))
    (setq after-change-functions
	  (delq 'crepulsion-change-func after-change-functions)))
  (force-mode-line-update))


(defun crepulsion-change-func (beg end repllen)
  (if (and crepulsion-mode
	   (eq this-command 'self-insert-command)
	   (eq repllen 0))
      (crepulsion-region beg end)))

;; how to handle composite chars?
(defun crepulsion-region (beg end)
  (save-excursion
    (goto-char end)
    (if (not (eolp))
	(forward-char))
    (if (not (eolp))
	(forward-char))
    (setq end (point))

    (goto-char beg)
    (if (not (bolp))
	(backward-char))
    (save-restriction
      (narrow-to-region (point) end)
      (goto-char (point-min))
      (forward-char)
      (while (not (eobp))
	(let ((b (crepulsion-forward-type (char-before)))
	      (a (crepulsion-backward-type (char-after))))
	  (if (and b a (not (eq b a)))
	      (insert " ")))
	(forward-char)))))

(defun crepulsion-forward-type (ch)
  (cond
   ((eq (char-syntax ch) ? )		;; whitespace
    nil)
   ((memq ch '(?! ?\" ?# ?$ ?% ?& ?' ?* ?+ ?, ?- ?. ?/ ?= ?? ?\\ ?^ ?_ ?| ?~
	       ?\( ?\[ ?{ ?<
	       ?$B!V(B ?$B!W(B ?$B!J(B ?$B!K(B ?$B!"(B ?$B!#(B ?$B!%(B ?$B!$(B ?$B&L(B))
    nil)
   ((eq (char-charset ch) 'japanese-jisx0208)
    'crepulsion-jp)
   (t
    'crepulsion-other)))

(defun crepulsion-backward-type (ch)
  (cond
   ((eq (char-syntax ch) ? )		;; whitespace
    nil)
   ((memq ch '(?! ?\" ?# ?$ ?% ?& ?' ?* ?+ ?, ?- ?. ?/ ?= ?? ?\\ ?^ ?_ ?| ?~
	       ?: ?; ?\) ?\] ?} ?>
	       ?$B!V(B ?$B!W(B ?$B!J(B ?$B!K(B ?$B!"(B ?$B!#(B ?$B!%(B ?$B!$(B))
    nil)
   ((eq (char-charset ch) 'japanese-jisx0208)
    'crepulsion-jp)
   (t
    'crepulsion-other)))


(provide 'crepulsion)
